import type { SyntaxNode } from "@lezer/common";
import type { LRParser } from "@lezer/lr";
import { type ComposedRange, type Node, type Token, type Language } from "../../../types";
import { Document } from "./Document";
type ContextInput = {
    source: SyntaxNode;
    document: Document;
};
type ContextGetter = (input: ContextInput, languageName: string) => string | null;
export type NodeReplacerInput = {
    node: Node;
    source: SyntaxNode;
    document: Document;
};
export type TokenHandlerInput = {
    source: SyntaxNode;
    document: Document;
    context: string[];
};
type NodeReplacer = (input: NodeReplacerInput) => Node | null;
type TokenHandler = (input: TokenHandlerInput) => Token | Node | null;
type MapperInput = {
    source: SyntaxNode;
};
type Mapper = (input: MapperInput) => string;
type LanguageAdapter = {
    name: Language;
    getParser: () => LRParser;
    contextMap: Record<string, ContextGetter>;
    syntaxMap: Record<string, Mapper>;
    tokenHandlerMap: Record<string, TokenHandler>;
    nodeReplacerMap: Record<string, NodeReplacer>;
    garbageHandler?: (input: TokenHandlerInput) => (Node | Token)[];
    postprocessor?: (root: Node) => void;
};
export declare function getHoles(source: SyntaxNode): {
    from: number;
    to: number;
}[];
export declare function mapSyntax(input: string[], to: string): Record<string, Mapper>;
export declare function mapContext(input: Record<string, string | ((input: MapperInput) => string)>, fallback?: string): (input: MapperInput) => string;
export declare function getContextFromChild(...types: string[]): ContextGetter;
export declare function prototypeNode(proto: {
    sx?: number;
    sy?: number;
    ex?: number;
    ey?: number;
    hash?: number;
    size?: number;
    type: string;
    context: string[];
    content?: (Node | Token)[];
}): Node;
export declare function parse(input: string, sliceFrom: number, sliceTo: number, context: string[], ranges: ComposedRange[], language: LanguageAdapter): Node[];
export {};
