import { z } from "zod";
import type { Language } from "./language";
import type { LanguageThemeMap } from "./theme";
export type { Theme } from "./theme";
export type { Language };
export type Coords = {
    sx: number;
    sy: number;
    ex: number;
    ey: number;
};
export type Diffable = Coords & {
    hash: number;
    context: string[];
    depth: number;
};
export type Token = Diffable & {
    text: string;
    type: string;
    range: number;
    language: Language;
};
export type InputRange = {
    from: number;
    to: number;
    data: Record<string, string>;
};
export type ComposedRange = InputRange & {
    hash: number;
};
export type CodeRange = Coords & ComposedRange;
export type Node = Diffable & {
    type: string;
    content: (Node | Token)[];
    size: number;
};
export declare function isNode(obj: Node | Token | null | undefined): obj is Node;
export declare function isToken(obj: Node | Token | null | undefined): obj is Token;
export type LanguageDefinition = {
    processor: (code: string, sliceFrom: number, sliceTo: number, inputContext: string[], ranges: ComposedRange[], flags: any) => Node[];
    themeMap: LanguageThemeMap;
};
declare const baseDecorationSchema: z.ZodObject<{
    data: z.ZodObject<{
        tagName: z.ZodOptional<z.ZodString>;
        attributes: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>;
    }, "strip", z.ZodTypeAny, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }>;
}, "strip", z.ZodTypeAny, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
}, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
}>;
type BaseDecoration = z.infer<typeof baseDecorationSchema>;
declare const inputGutterDecorationSchema: z.ZodObject<z.objectUtil.extendShape<{
    data: z.ZodObject<{
        tagName: z.ZodOptional<z.ZodString>;
        attributes: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>;
    }, "strip", z.ZodTypeAny, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }>;
}, {
    kind: z.ZodLiteral<"GUTTER">;
    class: z.ZodLiteral<"character">;
    text: z.ZodString;
    line: z.ZodNumber;
}>, "strip", z.ZodTypeAny, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    line: number;
    text: string;
    class: "character";
    kind: "GUTTER";
}, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    line: number;
    text: string;
    class: "character";
    kind: "GUTTER";
}>;
export type InputGutterDecoration = z.infer<typeof inputGutterDecorationSchema>;
declare const inputLineDecorationSchema: z.ZodObject<z.objectUtil.extendShape<{
    data: z.ZodObject<{
        tagName: z.ZodOptional<z.ZodString>;
        attributes: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>;
    }, "strip", z.ZodTypeAny, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }>;
}, {
    kind: z.ZodLiteral<"LINE">;
    class: z.ZodUnion<[z.ZodLiteral<"highlight">, z.ZodLiteral<"error">, z.ZodLiteral<"ok">, z.ZodLiteral<"custom1">, z.ZodLiteral<"custom2">, z.ZodLiteral<"custom3">]>;
    fromLine: z.ZodNumber;
    toLine: z.ZodNumber;
}>, "strip", z.ZodTypeAny, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    class: "error" | "highlight" | "ok" | "custom1" | "custom2" | "custom3";
    kind: "LINE";
    fromLine: number;
    toLine: number;
}, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    class: "error" | "highlight" | "ok" | "custom1" | "custom2" | "custom3";
    kind: "LINE";
    fromLine: number;
    toLine: number;
}>;
export type InputLineDecoration = z.infer<typeof inputLineDecorationSchema>;
declare const inputTextDecorationSchema: z.ZodObject<z.objectUtil.extendShape<{
    data: z.ZodObject<{
        tagName: z.ZodOptional<z.ZodString>;
        attributes: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>;
    }, "strip", z.ZodTypeAny, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }>;
}, {
    kind: z.ZodLiteral<"TEXT">;
    class: z.ZodUnion<[z.ZodLiteral<"highlight">, z.ZodLiteral<"error">, z.ZodLiteral<"ok">, z.ZodLiteral<"custom1">, z.ZodLiteral<"custom2">, z.ZodLiteral<"custom3">]>;
    from: z.ZodNumber;
    to: z.ZodNumber;
}>, "strip", z.ZodTypeAny, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    class: "error" | "highlight" | "ok" | "custom1" | "custom2" | "custom3";
    from: number;
    to: number;
    kind: "TEXT";
}, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    class: "error" | "highlight" | "ok" | "custom1" | "custom2" | "custom3";
    from: number;
    to: number;
    kind: "TEXT";
}>;
export type InputTextDecoration = z.infer<typeof inputTextDecorationSchema>;
export declare const inputDecorationSchema: z.ZodDiscriminatedUnion<"kind", [z.ZodObject<z.objectUtil.extendShape<{
    data: z.ZodObject<{
        tagName: z.ZodOptional<z.ZodString>;
        attributes: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>;
    }, "strip", z.ZodTypeAny, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }>;
}, {
    kind: z.ZodLiteral<"GUTTER">;
    class: z.ZodLiteral<"character">;
    text: z.ZodString;
    line: z.ZodNumber;
}>, "strip", z.ZodTypeAny, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    line: number;
    text: string;
    class: "character";
    kind: "GUTTER";
}, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    line: number;
    text: string;
    class: "character";
    kind: "GUTTER";
}>, z.ZodObject<z.objectUtil.extendShape<{
    data: z.ZodObject<{
        tagName: z.ZodOptional<z.ZodString>;
        attributes: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>;
    }, "strip", z.ZodTypeAny, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }>;
}, {
    kind: z.ZodLiteral<"LINE">;
    class: z.ZodUnion<[z.ZodLiteral<"highlight">, z.ZodLiteral<"error">, z.ZodLiteral<"ok">, z.ZodLiteral<"custom1">, z.ZodLiteral<"custom2">, z.ZodLiteral<"custom3">]>;
    fromLine: z.ZodNumber;
    toLine: z.ZodNumber;
}>, "strip", z.ZodTypeAny, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    class: "error" | "highlight" | "ok" | "custom1" | "custom2" | "custom3";
    kind: "LINE";
    fromLine: number;
    toLine: number;
}, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    class: "error" | "highlight" | "ok" | "custom1" | "custom2" | "custom3";
    kind: "LINE";
    fromLine: number;
    toLine: number;
}>, z.ZodObject<z.objectUtil.extendShape<{
    data: z.ZodObject<{
        tagName: z.ZodOptional<z.ZodString>;
        attributes: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodString>>;
    }, "strip", z.ZodTypeAny, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }, {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    }>;
}, {
    kind: z.ZodLiteral<"TEXT">;
    class: z.ZodUnion<[z.ZodLiteral<"highlight">, z.ZodLiteral<"error">, z.ZodLiteral<"ok">, z.ZodLiteral<"custom1">, z.ZodLiteral<"custom2">, z.ZodLiteral<"custom3">]>;
    from: z.ZodNumber;
    to: z.ZodNumber;
}>, "strip", z.ZodTypeAny, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    class: "error" | "highlight" | "ok" | "custom1" | "custom2" | "custom3";
    from: number;
    to: number;
    kind: "TEXT";
}, {
    data: {
        attributes?: Record<string, string> | undefined;
        tagName?: string | undefined;
    };
    class: "error" | "highlight" | "ok" | "custom1" | "custom2" | "custom3";
    from: number;
    to: number;
    kind: "TEXT";
}>]>;
export type InputDecoration = z.infer<typeof inputDecorationSchema>;
type ReifiedDecoration = Coords & Diffable;
export type GutterDecoration = InputGutterDecoration & ReifiedDecoration;
export type LineDecoration = InputLineDecoration & ReifiedDecoration;
export type TextDecoration = InputTextDecoration & ReifiedDecoration;
export type Decoration = GutterDecoration | LineDecoration | TextDecoration;
export type UniqueDecoration = Decoration & {
    id: string;
};
export type FrameMetadata = {
    renderedText: string;
    cols: number;
    rows: number;
};
export type EQL<T> = {
    readonly kind: "EQL";
    item: T;
};
export type ADD<T> = {
    readonly kind: "ADD";
    item: T;
};
export type DEL<T> = {
    readonly kind: "DEL";
    item: T;
};
export type MOV<T> = {
    readonly kind: "MOV";
    item: T;
    from: T;
};
export type DiffOp<T> = EQL<T> | ADD<T> | DEL<T> | MOV<T>;
export type BAD<T> = {
    readonly kind: "BAD";
    item: T;
};
export type BDE<T> = {
    readonly kind: "BDE";
    item: T;
    from: T;
};
export type ExtendedDiffOp<T> = DiffOp<T> | BAD<T> | BDE<T>;
export type TokenOutput = {
    id: number;
    text: string;
    type: string;
    language: Language;
};
type GutterDecorationOutput = BaseDecoration & {
    id: number;
    text: string;
    readonly kind: InputGutterDecoration["kind"];
    readonly class: InputGutterDecoration["class"];
};
export type LineDecorationOutput = BaseDecoration & {
    id: number;
    readonly kind: InputLineDecoration["kind"];
    readonly class: InputLineDecoration["class"];
};
type TextDecorationOutput = BaseDecoration & {
    id: number;
    readonly kind: InputTextDecoration["kind"];
    readonly class: InputTextDecoration["class"];
};
export type DecorationOutput = GutterDecorationOutput | LineDecorationOutput | TextDecorationOutput;
export type TextPosition = {
    x: number;
    y: number;
    a: number;
};
export type DecorationPosition = {
    x: number;
    y: number;
    w: number;
    h: number;
    a: number;
};
export type Frame = {
    isKeyframe: boolean;
    text: Map<number, TextPosition>;
    decoration: Map<number, DecorationPosition>;
};
export type Scene = {
    cols: number;
    rows: number;
    objects: {
        text: Map<number, TokenOutput>;
        decoration: Map<number, DecorationOutput>;
    };
    frames: Map<number, Frame>;
    metadata: Map<number, FrameMetadata>;
};
export type InputFrame = {
    code: string;
    ranges: InputRange[];
    decorations: InputDecoration[];
};
export type InputOptions = {
    language: Language;
    tabSize: number;
};
