# Animations

## Input: `fromStringsToScene(inputFrames, inputOptions)`

Turns frames of code into a `Scene` object that contains the animation information.

### Parameters

#### `inputFrames`

An array of objects conforming to the following TypeScript type:

```typescript
type InputOptions = {
  code: string;
  ranges: InputRange[];
  decorations: InputDecoration[];
};
```

Each object represents one keyframe in the resulting animation.

* `code`: Source code for the current frame
* `ranges`: Array. *Work in Progress; Ignore this for now.*
* `decorations`: Array. *Work in Progress; Ignore this for now.*

#### `inputOptions`

An object conforming to the following TypeScript type:

```typescript
type InputOptions = {
  language: Language;
  tabSize: number;
};
```

* `language`: Name of the [language](../languages/Readme.md) (`"css"`, `"javascript"`, `"typescript"`, `"html"`, or `"json"`) to parse the input as.
* `tabSize`: Sets the tab size. Must be a whole number.

### Return value

A scene object. Contains the animation information and some metadata. Best passed straight to `toAnimationHTML()`.

## Output: `toAnimationHTML(scene, animationOptions)`

Turns a scene and some animation options into a giant string of HTML and CSS that renders the animation.

### Parameters

#### `scene`

The animation information generated from `fromStringsToScene()`.

#### `animationOptions`

An object conforming to the following TypeScript type:

```typescript
type AnimationOptions = {
  rootClassSeed?: number;
  debugOverride?: boolean;
  theme?: Theme;
};
```

* `rootClassSeed`: Input used to generate a unique ID for the animation, which is required to namespace the CSS so that multiple animations can co-exist on one web page. Set this to something reasonably unique-ish, like an automatically incrementing ID or `Date.now()`. Must be a whole number. Optional, defaults to `0`.
* `debugOverride`: Can be set to true to turn off class name minification/obfuscation. Optional, defaults to `false`.
* `theme`: [Theme Object](./../styling/Theme.md) to use. Optional, defaults to the default theme.

### Return value

A string of HTML containing the animation, expressed as a `<div>` and a `<style>` element.

You can jump between keyframes by switching between the classes `frame0`, `frame1`, `frameN` etc. on the element with the class `cm-animation` or use [the Code.Movie runtime element](https://github.com/CodeMovie/code-movie-runtime).
