import { z } from "zod";
export { defaultTheme } from "./default";
declare const backdropSchema: z.ZodObject<{
    kind: z.ZodLiteral<"BACKDROP">;
    value: z.ZodUnion<[z.ZodObject<{
        kind: z.ZodLiteral<"COLOR">;
        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
    }, "strip", z.ZodTypeAny, {
        value: string;
        kind: "COLOR";
    }, {
        value: string;
        kind: "COLOR";
    }>, z.ZodObject<{
        kind: z.ZodLiteral<"LINEAR_GRADIENT">;
        stops: z.ZodArray<z.ZodObject<{
            color: z.ZodObject<{
                kind: z.ZodLiteral<"COLOR">;
                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
            }, "strip", z.ZodTypeAny, {
                value: string;
                kind: "COLOR";
            }, {
                value: string;
                kind: "COLOR";
            }>;
            position: z.ZodNumber;
        }, "strip", z.ZodTypeAny, {
            color: {
                value: string;
                kind: "COLOR";
            };
            position: number;
        }, {
            color: {
                value: string;
                kind: "COLOR";
            };
            position: number;
        }>, "many">;
        angle: z.ZodNumber;
    }, "strip", z.ZodTypeAny, {
        angle: number;
        kind: "LINEAR_GRADIENT";
        stops: {
            color: {
                value: string;
                kind: "COLOR";
            };
            position: number;
        }[];
    }, {
        angle: number;
        kind: "LINEAR_GRADIENT";
        stops: {
            color: {
                value: string;
                kind: "COLOR";
            };
            position: number;
        }[];
    }>]>;
}, "strip", z.ZodTypeAny, {
    value: {
        value: string;
        kind: "COLOR";
    } | {
        angle: number;
        kind: "LINEAR_GRADIENT";
        stops: {
            color: {
                value: string;
                kind: "COLOR";
            };
            position: number;
        }[];
    };
    kind: "BACKDROP";
}, {
    value: {
        value: string;
        kind: "COLOR";
    } | {
        angle: number;
        kind: "LINEAR_GRADIENT";
        stops: {
            color: {
                value: string;
                kind: "COLOR";
            };
            position: number;
        }[];
    };
    kind: "BACKDROP";
}>;
export type BackgroundSchema = z.infer<typeof backdropSchema>;
export declare const tokenThemeTypes: {
    readonly keyword: "keyword";
    readonly alternativeKeyword: "alternativeKeyword";
    readonly tag: "tag";
    readonly alternativeTag: "alternativeTag";
    readonly attribute: "attribute";
    readonly alternativeAttribute: "alternativeAttribute";
    readonly declaration: "declaration";
    readonly string: "string";
    readonly number: "number";
    readonly literal: "literal";
    readonly value: "value";
    readonly operator: "operator";
    readonly invocation: "invocation";
    readonly type: "type";
    readonly comment: "comment";
    readonly alternativeComment: "comment";
    readonly punctuation: "punctuation";
};
export type LanguageThemeColorNames = keyof typeof tokenThemeTypes;
export type LanguageThemeMap = Record<string, LanguageThemeColorNames>;
declare const underlineSchema: z.ZodObject<{
    kind: z.ZodLiteral<"UNDERLINE">;
    offsetY: z.ZodNumber;
    enabled: z.ZodBoolean;
    style: z.ZodEnum<["line", "wave", "squiggly"]>;
    color: z.ZodObject<{
        kind: z.ZodLiteral<"COLOR">;
        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
    }, "strip", z.ZodTypeAny, {
        value: string;
        kind: "COLOR";
    }, {
        value: string;
        kind: "COLOR";
    }>;
    scale: z.ZodNumber;
    width: z.ZodNumber;
}, "strip", z.ZodTypeAny, {
    width: number;
    color: {
        value: string;
        kind: "COLOR";
    };
    scale: number;
    style: "line" | "wave" | "squiggly";
    enabled: boolean;
    kind: "UNDERLINE";
    offsetY: number;
}, {
    width: number;
    color: {
        value: string;
        kind: "COLOR";
    };
    scale: number;
    style: "line" | "wave" | "squiggly";
    enabled: boolean;
    kind: "UNDERLINE";
    offsetY: number;
}>;
export type UnderlineThemeParams = z.infer<typeof underlineSchema>;
export declare const themeSchema: z.ZodObject<{
    version: z.ZodLiteral<0>;
    verticalAlign: z.ZodUnion<[z.ZodLiteral<"top">, z.ZodLiteral<"middle">]>;
    padding: z.ZodTuple<[z.ZodNumber, z.ZodNumber, z.ZodNumber, z.ZodNumber], null>;
    animation: z.ZodObject<{
        function: z.ZodUnion<[z.ZodLiteral<"ease">, z.ZodLiteral<"ease-in">, z.ZodLiteral<"ease-out">, z.ZodLiteral<"ease-in-out">, z.ZodLiteral<"linear">]>;
        duration: z.ZodNumber;
    }, "strip", z.ZodTypeAny, {
        function: "linear" | "ease" | "ease-in" | "ease-out" | "ease-in-out";
        duration: number;
    }, {
        function: "linear" | "ease" | "ease-in" | "ease-out" | "ease-in-out";
        duration: number;
    }>;
    layers: z.ZodObject<{
        lineNumbers: z.ZodObject<{
            enabled: z.ZodBoolean;
            typography: z.ZodObject<{
                color: z.ZodObject<{
                    kind: z.ZodLiteral<"COLOR">;
                    value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                }, "strip", z.ZodTypeAny, {
                    value: string;
                    kind: "COLOR";
                }, {
                    value: string;
                    kind: "COLOR";
                }>;
                weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
            }, "strip", z.ZodTypeAny, {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            }, {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            }>;
            horizontalMargin: z.ZodTuple<[z.ZodNumber, z.ZodNumber], null>;
        }, "strip", z.ZodTypeAny, {
            enabled: boolean;
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            };
            horizontalMargin: [number, number];
        }, {
            enabled: boolean;
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            };
            horizontalMargin: [number, number];
        }>;
        tokens: z.ZodObject<{
            typography: z.ZodIntersection<z.ZodObject<{
                color: z.ZodObject<{
                    kind: z.ZodLiteral<"COLOR">;
                    value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                }, "strip", z.ZodTypeAny, {
                    value: string;
                    kind: "COLOR";
                }, {
                    value: string;
                    kind: "COLOR";
                }>;
                weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
            }, "strip", z.ZodTypeAny, {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            }, {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            }>, z.ZodObject<{
                kind: z.ZodLiteral<"TYPOGRAPHY">;
                fontFamily: z.ZodString;
                lineHeight: z.ZodNumber;
                fontSizeFactor: z.ZodNumber;
            }, "strip", z.ZodTypeAny, {
                fontFamily: string;
                lineHeight: number;
                kind: "TYPOGRAPHY";
                fontSizeFactor: number;
            }, {
                fontFamily: string;
                lineHeight: number;
                kind: "TYPOGRAPHY";
                fontSizeFactor: number;
            }>>;
            colors: z.ZodObject<{
                string: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                number: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                keyword: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                alternativeKeyword: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                tag: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                alternativeTag: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                attribute: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                alternativeAttribute: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                declaration: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                literal: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                value: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                operator: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                invocation: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                type: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                comment: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                alternativeComment: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
                punctuation: z.ZodObject<{
                    color: z.ZodObject<{
                        kind: z.ZodLiteral<"COLOR">;
                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                    }, "strip", z.ZodTypeAny, {
                        value: string;
                        kind: "COLOR";
                    }, {
                        value: string;
                        kind: "COLOR";
                    }>;
                    weight: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"bold">]>;
                    style: z.ZodUnion<[z.ZodLiteral<"normal">, z.ZodLiteral<"italic">]>;
                }, "strip", z.ZodTypeAny, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }, {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                }>;
            }, "strip", z.ZodTypeAny, {
                string: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                number: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                type: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                value: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                literal: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                comment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                keyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeKeyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                tag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeTag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                attribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeAttribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                declaration: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                operator: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                invocation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeComment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                punctuation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
            }, {
                string: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                number: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                type: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                value: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                literal: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                comment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                keyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeKeyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                tag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeTag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                attribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeAttribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                declaration: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                operator: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                invocation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeComment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                punctuation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
            }>;
        }, "strip", z.ZodTypeAny, {
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            } & {
                fontFamily: string;
                lineHeight: number;
                kind: "TYPOGRAPHY";
                fontSizeFactor: number;
            };
            colors: {
                string: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                number: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                type: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                value: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                literal: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                comment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                keyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeKeyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                tag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeTag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                attribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeAttribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                declaration: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                operator: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                invocation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeComment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                punctuation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
            };
        }, {
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            } & {
                fontFamily: string;
                lineHeight: number;
                kind: "TYPOGRAPHY";
                fontSizeFactor: number;
            };
            colors: {
                string: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                number: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                type: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                value: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                literal: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                comment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                keyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeKeyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                tag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeTag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                attribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeAttribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                declaration: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                operator: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                invocation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeComment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                punctuation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
            };
        }>;
        decorations: z.ZodObject<{
            gutter: z.ZodObject<{
                character: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_GUTTER">;
                    offset: z.ZodTuple<[z.ZodNumber, z.ZodNumber], null>;
                }, "strip", z.ZodTypeAny, {
                    offset: [number, number];
                    kind: "DECORATION_GUTTER";
                }, {
                    offset: [number, number];
                    kind: "DECORATION_GUTTER";
                }>;
            }, "strip", z.ZodTypeAny, {
                character: {
                    offset: [number, number];
                    kind: "DECORATION_GUTTER";
                };
            }, {
                character: {
                    offset: [number, number];
                    kind: "DECORATION_GUTTER";
                };
            }>;
            line: z.ZodObject<{
                highlight: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_LINE">;
                    class: z.ZodLiteral<"highlight">;
                    foreground: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                    background: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }>;
                error: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_LINE">;
                    class: z.ZodLiteral<"error">;
                    foreground: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                    background: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "error";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "error";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }>;
                ok: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_LINE">;
                    class: z.ZodLiteral<"ok">;
                    foreground: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                    background: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }>;
                custom1: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_LINE">;
                    class: z.ZodLiteral<"custom1">;
                    foreground: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                    background: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }>;
                custom2: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_LINE">;
                    class: z.ZodLiteral<"custom2">;
                    foreground: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                    background: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }>;
                custom3: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_LINE">;
                    class: z.ZodLiteral<"custom3">;
                    foreground: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                    background: z.ZodObject<{
                        endLeft: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"gutter">, z.ZodLiteral<"scene">]>;
                        endRight: z.ZodUnion<[z.ZodLiteral<"tokens">, z.ZodLiteral<"scene">]>;
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }, {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }, {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                }>;
            }, "strip", z.ZodTypeAny, {
                error: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "error";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                highlight: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                ok: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom1: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom2: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom3: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
            }, {
                error: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "error";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                highlight: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                ok: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom1: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom2: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom3: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
            }>;
            text: z.ZodObject<{
                highlight: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_TEXT">;
                    class: z.ZodLiteral<"highlight">;
                    foreground: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                    background: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }>;
                error: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_TEXT">;
                    class: z.ZodLiteral<"error">;
                    foreground: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                    background: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "error";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "error";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }>;
                ok: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_TEXT">;
                    class: z.ZodLiteral<"ok">;
                    foreground: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                    background: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }>;
                custom1: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_TEXT">;
                    class: z.ZodLiteral<"custom1">;
                    foreground: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                    background: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }>;
                custom2: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_TEXT">;
                    class: z.ZodLiteral<"custom2">;
                    foreground: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                    background: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }>;
                custom3: z.ZodObject<{
                    kind: z.ZodLiteral<"DECORATION_TEXT">;
                    class: z.ZodLiteral<"custom3">;
                    foreground: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                    background: z.ZodObject<{
                        backdrop: z.ZodObject<{
                            kind: z.ZodLiteral<"BACKDROP">;
                            value: z.ZodUnion<[z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>, z.ZodObject<{
                                kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                                stops: z.ZodArray<z.ZodObject<{
                                    color: z.ZodObject<{
                                        kind: z.ZodLiteral<"COLOR">;
                                        value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                                    }, "strip", z.ZodTypeAny, {
                                        value: string;
                                        kind: "COLOR";
                                    }, {
                                        value: string;
                                        kind: "COLOR";
                                    }>;
                                    position: z.ZodNumber;
                                }, "strip", z.ZodTypeAny, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }, {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }>, "many">;
                                angle: z.ZodNumber;
                            }, "strip", z.ZodTypeAny, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }, {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            }>]>;
                        }, "strip", z.ZodTypeAny, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }, {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        }>;
                        underline: z.ZodObject<{
                            kind: z.ZodLiteral<"UNDERLINE">;
                            offsetY: z.ZodNumber;
                            enabled: z.ZodBoolean;
                            style: z.ZodEnum<["line", "wave", "squiggly"]>;
                            color: z.ZodObject<{
                                kind: z.ZodLiteral<"COLOR">;
                                value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                            }, "strip", z.ZodTypeAny, {
                                value: string;
                                kind: "COLOR";
                            }, {
                                value: string;
                                kind: "COLOR";
                            }>;
                            scale: z.ZodNumber;
                            width: z.ZodNumber;
                        }, "strip", z.ZodTypeAny, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }, {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        }>;
                    }, "strip", z.ZodTypeAny, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }, {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    }>;
                }, "strip", z.ZodTypeAny, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }, {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                }>;
            }, "strip", z.ZodTypeAny, {
                error: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "error";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                highlight: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                ok: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom1: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom2: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom3: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
            }, {
                error: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "error";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                highlight: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                ok: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom1: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom2: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom3: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
            }>;
        }, "strip", z.ZodTypeAny, {
            line: {
                error: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "error";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                highlight: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                ok: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom1: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom2: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom3: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
            };
            text: {
                error: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "error";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                highlight: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                ok: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom1: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom2: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom3: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
            };
            gutter: {
                character: {
                    offset: [number, number];
                    kind: "DECORATION_GUTTER";
                };
            };
        }, {
            line: {
                error: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "error";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                highlight: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                ok: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom1: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom2: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom3: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
            };
            text: {
                error: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "error";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                highlight: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                ok: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom1: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom2: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom3: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
            };
            gutter: {
                character: {
                    offset: [number, number];
                    kind: "DECORATION_GUTTER";
                };
            };
        }>;
        background: z.ZodObject<{
            backdrop: z.ZodObject<{
                kind: z.ZodLiteral<"BACKDROP">;
                value: z.ZodUnion<[z.ZodObject<{
                    kind: z.ZodLiteral<"COLOR">;
                    value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                }, "strip", z.ZodTypeAny, {
                    value: string;
                    kind: "COLOR";
                }, {
                    value: string;
                    kind: "COLOR";
                }>, z.ZodObject<{
                    kind: z.ZodLiteral<"LINEAR_GRADIENT">;
                    stops: z.ZodArray<z.ZodObject<{
                        color: z.ZodObject<{
                            kind: z.ZodLiteral<"COLOR">;
                            value: z.ZodUnion<[z.ZodString, z.ZodString]>;
                        }, "strip", z.ZodTypeAny, {
                            value: string;
                            kind: "COLOR";
                        }, {
                            value: string;
                            kind: "COLOR";
                        }>;
                        position: z.ZodNumber;
                    }, "strip", z.ZodTypeAny, {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }, {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }>, "many">;
                    angle: z.ZodNumber;
                }, "strip", z.ZodTypeAny, {
                    angle: number;
                    kind: "LINEAR_GRADIENT";
                    stops: {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }[];
                }, {
                    angle: number;
                    kind: "LINEAR_GRADIENT";
                    stops: {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }[];
                }>]>;
            }, "strip", z.ZodTypeAny, {
                value: {
                    value: string;
                    kind: "COLOR";
                } | {
                    angle: number;
                    kind: "LINEAR_GRADIENT";
                    stops: {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }[];
                };
                kind: "BACKDROP";
            }, {
                value: {
                    value: string;
                    kind: "COLOR";
                } | {
                    angle: number;
                    kind: "LINEAR_GRADIENT";
                    stops: {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }[];
                };
                kind: "BACKDROP";
            }>;
        }, "strip", z.ZodTypeAny, {
            backdrop: {
                value: {
                    value: string;
                    kind: "COLOR";
                } | {
                    angle: number;
                    kind: "LINEAR_GRADIENT";
                    stops: {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }[];
                };
                kind: "BACKDROP";
            };
        }, {
            backdrop: {
                value: {
                    value: string;
                    kind: "COLOR";
                } | {
                    angle: number;
                    kind: "LINEAR_GRADIENT";
                    stops: {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }[];
                };
                kind: "BACKDROP";
            };
        }>;
    }, "strip", z.ZodTypeAny, {
        background: {
            backdrop: {
                value: {
                    value: string;
                    kind: "COLOR";
                } | {
                    angle: number;
                    kind: "LINEAR_GRADIENT";
                    stops: {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }[];
                };
                kind: "BACKDROP";
            };
        };
        lineNumbers: {
            enabled: boolean;
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            };
            horizontalMargin: [number, number];
        };
        tokens: {
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            } & {
                fontFamily: string;
                lineHeight: number;
                kind: "TYPOGRAPHY";
                fontSizeFactor: number;
            };
            colors: {
                string: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                number: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                type: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                value: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                literal: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                comment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                keyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeKeyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                tag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeTag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                attribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeAttribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                declaration: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                operator: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                invocation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeComment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                punctuation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
            };
        };
        decorations: {
            line: {
                error: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "error";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                highlight: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                ok: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom1: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom2: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom3: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
            };
            text: {
                error: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "error";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                highlight: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                ok: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom1: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom2: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom3: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
            };
            gutter: {
                character: {
                    offset: [number, number];
                    kind: "DECORATION_GUTTER";
                };
            };
        };
    }, {
        background: {
            backdrop: {
                value: {
                    value: string;
                    kind: "COLOR";
                } | {
                    angle: number;
                    kind: "LINEAR_GRADIENT";
                    stops: {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }[];
                };
                kind: "BACKDROP";
            };
        };
        lineNumbers: {
            enabled: boolean;
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            };
            horizontalMargin: [number, number];
        };
        tokens: {
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            } & {
                fontFamily: string;
                lineHeight: number;
                kind: "TYPOGRAPHY";
                fontSizeFactor: number;
            };
            colors: {
                string: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                number: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                type: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                value: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                literal: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                comment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                keyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeKeyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                tag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeTag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                attribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeAttribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                declaration: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                operator: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                invocation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeComment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                punctuation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
            };
        };
        decorations: {
            line: {
                error: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "error";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                highlight: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                ok: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom1: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom2: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom3: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
            };
            text: {
                error: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "error";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                highlight: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                ok: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom1: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom2: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom3: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
            };
            gutter: {
                character: {
                    offset: [number, number];
                    kind: "DECORATION_GUTTER";
                };
            };
        };
    }>;
}, "strip", z.ZodTypeAny, {
    animation: {
        function: "linear" | "ease" | "ease-in" | "ease-out" | "ease-in-out";
        duration: number;
    };
    padding: [number, number, number, number];
    verticalAlign: "top" | "middle";
    version: 0;
    layers: {
        background: {
            backdrop: {
                value: {
                    value: string;
                    kind: "COLOR";
                } | {
                    angle: number;
                    kind: "LINEAR_GRADIENT";
                    stops: {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }[];
                };
                kind: "BACKDROP";
            };
        };
        lineNumbers: {
            enabled: boolean;
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            };
            horizontalMargin: [number, number];
        };
        tokens: {
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            } & {
                fontFamily: string;
                lineHeight: number;
                kind: "TYPOGRAPHY";
                fontSizeFactor: number;
            };
            colors: {
                string: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                number: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                type: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                value: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                literal: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                comment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                keyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeKeyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                tag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeTag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                attribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeAttribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                declaration: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                operator: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                invocation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeComment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                punctuation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
            };
        };
        decorations: {
            line: {
                error: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "error";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                highlight: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                ok: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom1: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom2: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom3: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
            };
            text: {
                error: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "error";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                highlight: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                ok: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom1: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom2: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom3: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
            };
            gutter: {
                character: {
                    offset: [number, number];
                    kind: "DECORATION_GUTTER";
                };
            };
        };
    };
}, {
    animation: {
        function: "linear" | "ease" | "ease-in" | "ease-out" | "ease-in-out";
        duration: number;
    };
    padding: [number, number, number, number];
    verticalAlign: "top" | "middle";
    version: 0;
    layers: {
        background: {
            backdrop: {
                value: {
                    value: string;
                    kind: "COLOR";
                } | {
                    angle: number;
                    kind: "LINEAR_GRADIENT";
                    stops: {
                        color: {
                            value: string;
                            kind: "COLOR";
                        };
                        position: number;
                    }[];
                };
                kind: "BACKDROP";
            };
        };
        lineNumbers: {
            enabled: boolean;
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            };
            horizontalMargin: [number, number];
        };
        tokens: {
            typography: {
                color: {
                    value: string;
                    kind: "COLOR";
                };
                style: "normal" | "italic";
                weight: "bold" | "normal";
            } & {
                fontFamily: string;
                lineHeight: number;
                kind: "TYPOGRAPHY";
                fontSizeFactor: number;
            };
            colors: {
                string: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                number: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                type: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                value: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                literal: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                comment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                keyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeKeyword: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                tag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeTag: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                attribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeAttribute: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                declaration: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                operator: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                invocation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                alternativeComment: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
                punctuation: {
                    color: {
                        value: string;
                        kind: "COLOR";
                    };
                    style: "normal" | "italic";
                    weight: "bold" | "normal";
                };
            };
        };
        decorations: {
            line: {
                error: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "error";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                highlight: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                ok: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom1: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom2: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
                custom3: {
                    background: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_LINE";
                    foreground: {
                        endLeft: "tokens" | "gutter" | "scene";
                        endRight: "tokens" | "scene";
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                    };
                };
            };
            text: {
                error: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "error";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                highlight: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "highlight";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                ok: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "ok";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom1: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom1";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom2: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom2";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
                custom3: {
                    background: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                    class: "custom3";
                    kind: "DECORATION_TEXT";
                    foreground: {
                        backdrop: {
                            value: {
                                value: string;
                                kind: "COLOR";
                            } | {
                                angle: number;
                                kind: "LINEAR_GRADIENT";
                                stops: {
                                    color: {
                                        value: string;
                                        kind: "COLOR";
                                    };
                                    position: number;
                                }[];
                            };
                            kind: "BACKDROP";
                        };
                        underline: {
                            width: number;
                            color: {
                                value: string;
                                kind: "COLOR";
                            };
                            scale: number;
                            style: "line" | "wave" | "squiggly";
                            enabled: boolean;
                            kind: "UNDERLINE";
                            offsetY: number;
                        };
                    };
                };
            };
            gutter: {
                character: {
                    offset: [number, number];
                    kind: "DECORATION_GUTTER";
                };
            };
        };
    };
}>;
export type Theme = z.infer<typeof themeSchema>;
export declare function migrateTheme(input: unknown): Theme;
